@@ -1,17 +1,27 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2006-2015 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
+# Copyright (c) 2006-2016 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
+# Copyright (c) 2010 Daniel Harding <dharding@gmail.com>
 # Copyright (c) 2012-2014 Google, Inc.
-# Copyright (c) 2013-2016 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2013-2017 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2014 Brett Cannon <brett@python.org>
-# Copyright (c) 2015 Radu Ciorba <radu@devrandom.ro>
+# Copyright (c) 2014 Arun Persaud <arun@nubati.net>
+# Copyright (c) 2015 Nick Bastin <nick.bastin@gmail.com>
 # Copyright (c) 2015 Michael Kefeder <oss@multiwave.ch>
 # Copyright (c) 2015 Dmitry Pribysh <dmand@yandex.ru>
 # Copyright (c) 2015 Stephane Wirtel <stephane@wirtel.be>
-# Copyright (c) 2015 Nick Bastin <nick.bastin@gmail.com>
-# Copyright (c) 2016 Alex Jurkiewicz <alex@jurkiewi.cz>
-# Copyright (c) 2016 Yannack <yannack@users.noreply.github.com>
-# Copyright (c) 2016 Laura Médioni <lmedioni@logilab.fr>
+# Copyright (c) 2015 Cosmin Poieana <cmin@ropython.org>
+# Copyright (c) 2015 Florian Bruhin <me@the-compiler.org>
+# Copyright (c) 2015 Radu Ciorba <radu@devrandom.ro>
+# Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
+# Copyright (c) 2016-2017 Łukasz Rogalski <rogalski.91@gmail.com>
+# Copyright (c) 2016 Glenn Matthews <glenn@e-dad.net>
+# Copyright (c) 2016 Elias Dorneles <eliasdorneles@gmail.com>
 # Copyright (c) 2016 Ashley Whetter <ashley@awhetter.co.uk>
+# Copyright (c) 2016 Yannack <yannack@users.noreply.github.com>
+# Copyright (c) 2016 Jakub Wilk <jwilk@jwilk.net>
+# Copyright (c) 2016 Alex Jurkiewicz <alex@jurkiewi.cz>
+# Copyright (c) 2017 ttenhoeve-aa <ttenhoeve@appannie.com>
+# Copyright (c) 2017 hippo91 <guillaume.peillex@gmail.com>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
@@ -35,16 +45,85 @@ from pylint import exceptions
 from pylint import interfaces
 from pylint.checkers import utils
 from pylint import reporters
+from pylint.checkers.utils import get_node_last_lineno
 from pylint.reporters.ureports import nodes as reporter_nodes
+import pylint.utils as lint_utils
 
 
-# regex for class/function/variable/constant name
-CLASS_NAME_RGX = re.compile('[A-Z_][a-zA-Z0-9]+$')
-MOD_NAME_RGX = re.compile('(([a-z_][a-z0-9_]*)|([A-Z][a-zA-Z0-9]+))$')
-CONST_NAME_RGX = re.compile('(([A-Z_][A-Z0-9_]*)|(__.*__))$')
-COMP_VAR_RGX = re.compile('[A-Za-z_][A-Za-z0-9_]*$')
-DEFAULT_NAME_RGX = re.compile('(([a-z][a-z0-9_]{2,30})|(_[a-z0-9_]*))$')
-CLASS_ATTRIBUTE_RGX = re.compile(r'([A-Za-z_][A-Za-z0-9_]{2,30}|(__.*__))$')
+class NamingStyle(object):
+    # It may seem counterintuitive that single naming style
+    # has multiple "accepted" forms of regular expressions,
+    # but we need to special-case stuff like dunder names
+    # in method names.
+    CLASS_NAME_RGX = None
+    MOD_NAME_RGX = None
+    CONST_NAME_RGX = None
+    COMP_VAR_RGX = None
+    DEFAULT_NAME_RGX = None
+    CLASS_ATTRIBUTE_RGX = None
+
+    @classmethod
+    def get_regex(cls, name_type):
+        return {
+            'module': cls.MOD_NAME_RGX,
+            'const': cls.CONST_NAME_RGX,
+            'class': cls.CLASS_NAME_RGX,
+            'function': cls.DEFAULT_NAME_RGX,
+            'method': cls.DEFAULT_NAME_RGX,
+            'attr': cls.DEFAULT_NAME_RGX,
+            'argument': cls.DEFAULT_NAME_RGX,
+            'variable': cls.DEFAULT_NAME_RGX,
+            'class_attribute': cls.CLASS_ATTRIBUTE_RGX,
+            'inlinevar': cls.COMP_VAR_RGX,
+        }[name_type]
+
+
+class SnakeCaseStyle(NamingStyle):
+    CLASS_NAME_RGX = re.compile('[a-z_][a-z0-9_]+$')
+    MOD_NAME_RGX = re.compile('([a-z_][a-z0-9_]*)$')
+    CONST_NAME_RGX = re.compile('(([a-z_][a-z0-9_]*)|(__.*__))$')
+    COMP_VAR_RGX = re.compile('[a-z_][a-z0-9_]*$')
+    DEFAULT_NAME_RGX = re.compile('(([a-z_][a-z0-9_]{2,30})|(_[a-z0-9_]*)|(__[a-z][a-z0-9_]+__))$')
+    CLASS_ATTRIBUTE_RGX = re.compile(r'(([a-z_][a-z0-9_]{2,30}|(__.*__)))$')
+
+
+class CamelCaseStyle(NamingStyle):
+    CLASS_NAME_RGX = re.compile('[a-z_][a-zA-Z0-9]+$')
+    MOD_NAME_RGX = re.compile('([a-z_][a-zA-Z0-9]*)$')
+    CONST_NAME_RGX = re.compile('(([a-z_][A-Za-z0-9]*)|(__.*__))$')
+    COMP_VAR_RGX = re.compile('[a-z_][A-Za-z0-9]*$')
+    DEFAULT_NAME_RGX = re.compile('(([a-z_][a-zA-Z0-9]{2,30})|(__[a-z][a-zA-Z0-9_]+__))$')
+    CLASS_ATTRIBUTE_RGX = re.compile(r'([a-z_][A-Za-z0-9]{2,30}|(__.*__))$')
+
+
+class PascalCaseStyle(NamingStyle):
+    CLASS_NAME_RGX = re.compile('[A-Z_][a-zA-Z0-9]+$')
+    MOD_NAME_RGX = re.compile('[A-Z_][a-zA-Z0-9]+$')
+    CONST_NAME_RGX = re.compile('(([A-Z_][A-Za-z0-9]*)|(__.*__))$')
+    COMP_VAR_RGX = re.compile('[A-Z_][a-zA-Z0-9]+$')
+    DEFAULT_NAME_RGX = re.compile('[A-Z_][a-zA-Z0-9]{2,30}$|(__[a-z][a-zA-Z0-9_]+__)$')
+    CLASS_ATTRIBUTE_RGX = re.compile('[A-Z_][a-zA-Z0-9]{2,30}$')
+
+
+class UpperCaseStyle(NamingStyle):
+    CLASS_NAME_RGX = re.compile('[A-Z_][A-Z0-9_]+$')
+    MOD_NAME_RGX = re.compile('[A-Z_][A-Z0-9_]+$')
+    CONST_NAME_RGX = re.compile('(([A-Z_][A-Z0-9_]*)|(__.*__))$')
+    COMP_VAR_RGX = re.compile('[A-Z_][A-Z0-9_]+$')
+    DEFAULT_NAME_RGX = re.compile('([A-Z_][A-Z0-9_]{2,30})|(__[a-z][a-zA-Z0-9_]+__)$')
+    CLASS_ATTRIBUTE_RGX = re.compile('[A-Z_][A-Z0-9_]{2,30}$')
+
+
+class AnyStyle(NamingStyle):
+    @classmethod
+    def get_regex(cls, name_type):
+        return re.compile('.*')
+
+
+NAMING_STYLES = {'snake_case': SnakeCaseStyle, 'camelCase': CamelCaseStyle,
+                 'PascalCase': PascalCaseStyle, 'UPPER_CASE': UpperCaseStyle,
+                 'any': AnyStyle}
+
 # do not require a doc string on private/system methods
 NO_REQUIRED_DOC_RGX = re.compile('^_')
 REVERSED_PROTOCOL_METHOD = '__reversed__'
@@ -62,7 +141,7 @@ PY3K = sys.version_info >= (3, 0)
 PY35 = sys.version_info >= (3, 5)
 
 # Name categories that are always consistent with all naming conventions.
-EXEMPT_NAME_CATEGORIES = set(('exempt', 'ignore'))
+EXEMPT_NAME_CATEGORIES = {'exempt', 'ignore'}
 
 # A mapping from builtin-qname -> symbol, to be used when generating messages
 # about dangerous default values as arguments
@@ -72,10 +151,9 @@ DEFAULT_ARGUMENT_SYMBOLS = dict(
 )
 REVERSED_COMPS = {'<': '>', '<=': '>=', '>': '<', '>=': '<='}
 
-del re
 
 def _redefines_import(node):
-    """ Detect that the given node (AssName) is inside an
+    """ Detect that the given node (AssignName) is inside an
     exception handler and redefines an import from the tryexcept body.
     Returns True if the node redefines an import, False otherwise.
     """
@@ -94,6 +172,7 @@ def _redefines_import(node):
                 return True
     return False
 
+
 def in_loop(node):
     """return True if the node is inside a kind of for loop"""
     parent = node.parent
@@ -104,6 +183,7 @@ def in_loop(node):
         parent = parent.parent
     return False
 
+
 def in_nested_list(nested_list, obj):
     """return true if the object is an element of <nested_list> or of a nested
     list
@@ -116,22 +196,49 @@ def in_nested_list(nested_list, obj):
             return True
     return False
 
-def _loop_exits_early(loop):
-    """Returns true if a loop has a break statement in its body."""
+
+def _get_break_loop_node(break_node):
+    """
+    Returns the loop node that holds the break node in arguments.
+
+    Args:
+        break_node (astroid.Break): the break node of interest.
+
+    Returns:
+        astroid.For or astroid.While: the loop node holding the break node.
+    """
     loop_nodes = (astroid.For, astroid.While)
-    # Loop over body explicitly to avoid matching break statements
-    # in orelse.
-    for child in loop.body:
-        if isinstance(child, loop_nodes):
-            # break statement may be in orelse of child loop.
-            # pylint: disable=superfluous-parens
-            for orelse in (child.orelse or ()):
-                for _ in orelse.nodes_of_class(astroid.Break, skip_klass=loop_nodes):
-                    return True
-            continue
-        for _ in child.nodes_of_class(astroid.Break, skip_klass=loop_nodes):
-            return True
-    return False
+    parent = break_node.parent
+    while not isinstance(parent, loop_nodes) or break_node in getattr(parent, 'orelse', []):
+        parent = parent.parent
+        if parent is None:
+            break
+    return parent
+
+
+def _loop_exits_early(loop):
+    """
+    Returns true if a loop may ends up in a break statement.
+
+    Args:
+        loop (astroid.For, astroid.While): the loop node inspected.
+
+    Returns:
+        bool: True if the loop may ends up in a break statement, False otherwise.
+    """
+    loop_nodes = (astroid.For, astroid.While)
+    definition_nodes = (astroid.FunctionDef, astroid.ClassDef)
+    inner_loop_nodes = [
+        _node for _node in loop.nodes_of_class(loop_nodes,
+                                               skip_klass=definition_nodes)
+        if _node != loop
+    ]
+    return any(
+        _node for _node in loop.nodes_of_class(astroid.Break,
+                                               skip_klass=definition_nodes)
+        if _get_break_loop_node(_node) not in inner_loop_nodes
+    )
+
 
 def _is_multi_naming_match(match, node_type, confidence):
     return (match is not None and
@@ -246,6 +353,7 @@ def report_by_type_stats(sect, stats, old_stats):
                   nice_stats[node_type].get('percent_badname', '0'))
     sect.append(reporter_nodes.Table(children=lines, cols=6, rheaders=1))
 
+
 def redefined_by_decorator(node):
     """return True if the object is a method redefined via decorator.
 
@@ -267,6 +375,7 @@ class _BasicChecker(checkers.BaseChecker):
     __implements__ = interfaces.IAstroidChecker
     name = 'basic'
 
+
 class BasicErrorChecker(_BasicChecker):
     msgs = {
         'E0100': ('__init__ method is a generator',
@@ -342,6 +451,11 @@ class BasicErrorChecker(_BasicChecker):
                   'Emitted when a nonlocal variable does not have an attached '
                   'name somewhere in the parent scopes',
                   {'minversion': (3, 0)}),
+        'E0118': ("Name %r is used prior to global declaration",
+                  'used-prior-global-declaration',
+                  'Emitted when a name is used prior a global declaration, '
+                  'which results in an error since Python 3.6.',
+                  {'minversion': (3, 6)}),
         }
 
     @utils.check_messages('function-redefined')
@@ -381,9 +495,11 @@ class BasicErrorChecker(_BasicChecker):
 
     @utils.check_messages('init-is-generator', 'return-in-init',
                           'function-redefined', 'return-arg-in-generator',
-                          'duplicate-argument-name', 'nonlocal-and-global')
+                          'duplicate-argument-name', 'nonlocal-and-global',
+                          'used-prior-global-declaration')
     def visit_functiondef(self, node):
         self._check_nonlocal_and_global(node)
+        self._check_name_used_prior_global(node)
         if (not redefined_by_decorator(node) and
                 not utils.is_registered_in_singledispatch_function(node)):
             self._check_redefinition(node.is_method() and 'method' or 'function', node)
@@ -397,7 +513,7 @@ class BasicErrorChecker(_BasicChecker):
             else:
                 values = [r.value for r in returns]
                 # Are we returning anything but None from constructors
-                if [v for v in values if not utils.is_none(v)]:
+                if any(v for v in values if not utils.is_none(v)):
                     self.add_message('return-in-init', node=node)
         elif node.is_generator():
             # make sure we don't mix non-None returns and yields
@@ -417,6 +533,29 @@ class BasicErrorChecker(_BasicChecker):
 
     visit_asyncfunctiondef = visit_functiondef
 
+    def _check_name_used_prior_global(self, node):
+
+        scope_globals = {
+            name: child
+            for child in node.nodes_of_class(astroid.Global)
+            for name in child.names
+            if child.scope() is node
+        }
+
+        for node_name in node.nodes_of_class(astroid.Name):
+            if node_name.scope() is not node:
+                continue
+
+            name = node_name.name
+            corresponding_global = scope_globals.get(name)
+            if not corresponding_global:
+                continue
+
+            global_lineno = corresponding_global.fromlineno
+            if global_lineno and global_lineno > node_name.fromlineno:
+                self.add_message('used-prior-global-declaration',
+                                 node=node_name, args=(name, ))
+
     def _check_nonlocal_and_global(self, node):
         """Check that a name is both nonlocal and global."""
         def same_scope(current):
@@ -476,19 +615,20 @@ class BasicErrorChecker(_BasicChecker):
             if current_scope.parent is None:
                 break
 
-            if not isinstance(current_scope, astroid.FunctionDef):
+            if not isinstance(current_scope, (astroid.ClassDef, astroid.FunctionDef)):
                 self.add_message('nonlocal-without-binding', args=(name, ),
                                  node=node)
                 return
-            else:
-                if name not in current_scope.locals:
-                    current_scope = current_scope.parent.scope()
-                    continue
-                else:
-                    # Okay, found it.
-                    return
 
-        self.add_message('nonlocal-without-binding', args=(name, ), node=node)
+            if name not in current_scope.locals:
+                current_scope = current_scope.parent.scope()
+                continue
+
+            # Okay, found it.
+            return
+
+        if not isinstance(current_scope, astroid.FunctionDef):
+            self.add_message('nonlocal-without-binding', args=(name, ), node=node)
 
     @utils.check_messages('nonlocal-without-binding')
     def visit_nonlocal(self, node):
@@ -501,10 +641,12 @@ class BasicErrorChecker(_BasicChecker):
         abc.ABCMeta as metaclass.
         """
         try:
-            infered = next(node.func.infer())
+            for inferred in node.func.infer():
+                self._check_inferred_class_is_abstract(inferred, node)
         except astroid.InferenceError:
             return
 
+    def _check_inferred_class_is_abstract(self, infered, node):
         if not isinstance(infered, astroid.ClassDef):
             return
 
@@ -570,11 +712,14 @@ class BasicErrorChecker(_BasicChecker):
         """check for redefinition of a function / method / class name"""
         defined_self = node.parent.frame()[node.name]
         if defined_self is not node and not astroid.are_exclusive(node, defined_self):
+            dummy_variables_rgx = lint_utils.get_global_option(
+                self, 'dummy-variables-rgx', default=None)
+            if dummy_variables_rgx and dummy_variables_rgx.match(defined_self.name):
+                return
             self.add_message('function-redefined', node=node,
                              args=(redeftype, defined_self.fromlineno))
 
 
-
 class BasicChecker(_BasicChecker):
     """checks for :
     * doc strings
@@ -704,7 +849,7 @@ functions, methods
         # These nodes are excepted, since they are not constant
         # values, requiring a computation to happen. The only type
         # of node in this list which doesn't have this property is
-        # Getattr, which is excepted because the conditional statement
+        # Attribute, which is excepted because the conditional statement
         # can be used to verify that the attribute was set inside a class,
         # which is definitely a valid use case.
         except_nodes = (astroid.Attribute, astroid.Call,
@@ -935,7 +1080,7 @@ functions, methods
 
     @utils.check_messages('eval-used', 'exec-used', 'bad-reversed-sequence')
     def visit_call(self, node):
-        """visit a CallFunc node -> check if this is not a blacklisted builtin
+        """visit a Call node -> check if this is not a blacklisted builtin
         call and check for * or ** use
         """
         if isinstance(node.func, astroid.Name):
@@ -1080,43 +1225,73 @@ functions, methods
                         self.add_message('confusing-with-statement', node=node)
 
 
-_NAME_TYPES = {
-    'module': (MOD_NAME_RGX, 'module'),
-    'const': (CONST_NAME_RGX, 'constant'),
-    'class': (CLASS_NAME_RGX, 'class'),
-    'function': (DEFAULT_NAME_RGX, 'function'),
-    'method': (DEFAULT_NAME_RGX, 'method'),
-    'attr': (DEFAULT_NAME_RGX, 'attribute'),
-    'argument': (DEFAULT_NAME_RGX, 'argument'),
-    'variable': (DEFAULT_NAME_RGX, 'variable'),
-    'class_attribute': (CLASS_ATTRIBUTE_RGX, 'class attribute'),
-    'inlinevar': (COMP_VAR_RGX, 'inline iteration'),
+KNOWN_NAME_TYPES = {
+    "module", "const", "class", "function", "method", "attr",
+    "argument", "variable", "class_attribute", "inlinevar"
 }
 
+
+HUMAN_READABLE_TYPES = {
+    'module': 'module',
+    'const': 'constant',
+    'class': 'class',
+    'function': 'function',
+    'method':  'method',
+    'attr': 'attribute',
+    'argument': 'argument',
+    'variable': 'variable',
+    'class_attribute': 'class attribute',
+    'inlinevar': 'inline iteration',
+}
+
+DEFAULT_NAMING_STYLES = {
+    "module": "snake_case",
+    "const": "UPPER_CASE",
+    "class": "PascalCase",
+    "function": "snake_case",
+    "method": "snake_case",
+    "attr": "snake_case",
+    "argument": "snake_case",
+    "variable": "snake_case",
+    "class_attribute": "any",
+    "inlinevar": "any",
+}
+
+
 def _create_naming_options():
     name_options = []
-    for name_type, (rgx, human_readable_name) in six.iteritems(_NAME_TYPES):
+    for name_type in KNOWN_NAME_TYPES:
+        human_readable_name = HUMAN_READABLE_TYPES[name_type]
+        default_style = DEFAULT_NAMING_STYLES[name_type]
         name_type = name_type.replace('_', '-')
         name_options.append((
-            '%s-rgx' % (name_type,),
-            {'default': rgx, 'type': 'regexp', 'metavar': '<regexp>',
-             'help': 'Regular expression matching correct %s names' % (human_readable_name,)}))
+            '%s-naming-style' % (name_type,),
+            {'default': default_style,
+             'type': 'choice', 'choices': list(NAMING_STYLES.keys()), 'metavar': '<style>',
+             'help': 'Naming style matching correct %s names' % (human_readable_name,)}),)
         name_options.append((
-            '%s-name-hint' % (name_type,),
-            {'default': rgx.pattern, 'type': 'string', 'metavar': '<string>',
-             'help': 'Naming hint for %s names' % (human_readable_name,)}))
+            '%s-rgx' % (name_type,),
+            {'default': None, 'type': 'regexp', 'metavar': '<regexp>',
+             'help': 'Regular expression matching correct %s names. Overrides %s-naming-style'
+                     % (human_readable_name, name_type,)}))
     return tuple(name_options)
 
+
 class NameChecker(_BasicChecker):
+
     msgs = {
         'C0102': ('Black listed name "%s"',
                   'blacklisted-name',
                   'Used when the name is listed in the black list (unauthorized '
                   'names).'),
-        'C0103': ('Invalid %s name "%s"%s',
+        'C0103': ('%s name "%s" doesn\'t conform to %s',
                   'invalid-name',
-                  'Used when the name doesn\'t match the regular expression '
+                  'Used when the name doesn\'t conform to naming rules '
                   'associated to its type (constant, variable, class...).'),
+        'W0111': ('Name %s will become a keyword in Python %s',
+                  'assign-to-new-keyword',
+                  'Used when assignment will become invalid in future '
+                  'Python release due to introducing new keyword'),
     }
 
     options = (('good-names',
@@ -1139,7 +1314,7 @@ class NameChecker(_BasicChecker):
                            ' allow several styles.')}
                ),
                ('include-naming-hint',
-                {'default': False, 'type' : 'yn', 'metavar' : '<y_or_n>',
+                {'default': False, 'type': 'yn', 'metavar': '<y_or_n>',
                  'help': 'Include a hint for the correct naming format with invalid-name'}
                ),
                ('property-classes',
@@ -1152,12 +1327,18 @@ class NameChecker(_BasicChecker):
                ),
               ) + _create_naming_options()
 
+    KEYWORD_ONSET = {
+        (3, 0): {'True', 'False'},
+        (3, 7): {'async', 'await'}
+    }
 
     def __init__(self, linter):
         _BasicChecker.__init__(self, linter)
         self._name_category = {}
         self._name_group = {}
         self._bad_names = {}
+        self._name_regexps = {}
+        self._name_hints = {}
 
     def open(self):
         self.stats = self.linter.add_stats(badname_module=0,
@@ -1172,6 +1353,32 @@ class NameChecker(_BasicChecker):
             for name_type in group.split(':'):
                 self._name_group[name_type] = 'group_%s' % (group,)
 
+        regexps, hints = self._create_naming_rules()
+        self._name_regexps = regexps
+        self._name_hints = hints
+
+    def _create_naming_rules(self):
+        regexps = {}
+        hints = {}
+
+        for name_type in KNOWN_NAME_TYPES:
+            naming_style_option_name = "%s_naming_style" % (name_type,)
+            naming_style_name = getattr(self.config, naming_style_option_name)
+
+            regexps[name_type] = NAMING_STYLES[naming_style_name].get_regex(name_type)
+
+            custom_regex_setting_name = "%s_rgx" % (name_type, )
+            custom_regex = getattr(self.config, custom_regex_setting_name, None)
+            if custom_regex is not None:
+                regexps[name_type] = custom_regex
+
+            if custom_regex is not None:
+                hints[name_type] = "%r pattern" % custom_regex.pattern
+            else:
+                hints[name_type] = "%s naming style" % naming_style_name
+
+        return regexps, hints
+
     @utils.check_messages('blacklisted-name', 'invalid-name')
     def visit_module(self, node):
         self._check_name('module', node.name.split('.')[-1], node)
@@ -1195,17 +1402,19 @@ class NameChecker(_BasicChecker):
             for args in warnings:
                 self._raise_name_warning(*args)
 
-    @utils.check_messages('blacklisted-name', 'invalid-name')
+    @utils.check_messages('blacklisted-name', 'invalid-name', 'assign-to-new-keyword')
     def visit_classdef(self, node):
+        self._check_assign_to_new_keyword_violation(node.name, node)
         self._check_name('class', node.name, node)
         for attr, anodes in six.iteritems(node.instance_attrs):
             if not any(node.instance_attr_ancestors(attr)):
                 self._check_name('attr', attr, anodes[0])
 
-    @utils.check_messages('blacklisted-name', 'invalid-name')
+    @utils.check_messages('blacklisted-name', 'invalid-name', 'assign-to-new-keyword')
     def visit_functiondef(self, node):
         # Do not emit any warnings if the method is just an implementation
         # of a base class method.
+        self._check_assign_to_new_keyword_violation(node.name, node)
         confidence = interfaces.HIGH
         if node.is_method():
             if utils.overrides_a_method(node.parent.frame(), node.name):
@@ -1228,9 +1437,10 @@ class NameChecker(_BasicChecker):
         for name in node.names:
             self._check_name('const', name, node)
 
-    @utils.check_messages('blacklisted-name', 'invalid-name')
+    @utils.check_messages('blacklisted-name', 'invalid-name', 'assign-to-new-keyword')
     def visit_assignname(self, node):
         """check module level assigned names"""
+        self._check_assign_to_new_keyword_violation(node.name, node)
         frame = node.frame()
         ass_type = node.assign_type()
         if isinstance(ass_type, astroid.Comprehension):
@@ -1267,11 +1477,17 @@ class NameChecker(_BasicChecker):
         return self._name_group.get(node_type, node_type)
 
     def _raise_name_warning(self, node, node_type, name, confidence):
-        type_label = _NAME_TYPES[node_type][1]
-        hint = ''
+        type_label = HUMAN_READABLE_TYPES[node_type]
+        hint = self._name_hints[node_type]
         if self.config.include_naming_hint:
-            hint = ' (hint: %s)' % (getattr(self.config, node_type + '_name_hint'))
-        self.add_message('invalid-name', node=node, args=(type_label, name, hint),
+            hint += " (%r pattern)" % self._name_regexps[node_type].pattern
+        args = (
+            type_label.capitalize(),
+            name,
+            hint
+        )
+
+        self.add_message('invalid-name', node=node, args=args,
                          confidence=confidence)
         self.stats['badname_' + node_type] += 1
 
@@ -1287,7 +1503,7 @@ class NameChecker(_BasicChecker):
             self.stats['badname_' + node_type] += 1
             self.add_message('blacklisted-name', node=node, args=name)
             return
-        regexp = getattr(self.config, node_type + '_rgx')
+        regexp = self._name_regexps[node_type]
         match = regexp.match(name)
 
         if _is_multi_naming_match(match, node_type, confidence):
@@ -1299,6 +1515,22 @@ class NameChecker(_BasicChecker):
         if match is None:
             self._raise_name_warning(node, node_type, name, confidence)
 
+    def _check_assign_to_new_keyword_violation(self, name, node):
+        keyword_first_version = self._name_became_keyword_in_version(
+            name, self.KEYWORD_ONSET
+        )
+        if keyword_first_version is not None:
+            self.add_message('assign-to-new-keyword',
+                             node=node, args=(name, keyword_first_version),
+                             confidence=interfaces.HIGH)
+
+    @staticmethod
+    def _name_became_keyword_in_version(name, rules):
+        for version, keywords in rules.items():
+            if name in keywords and sys.version_info < version:
+                return '.'.join(map(str, version))
+        return None
+
 
 class DocStringChecker(_BasicChecker):
     msgs = {
@@ -1327,12 +1559,12 @@ class DocStringChecker(_BasicChecker):
                ),
               )
 
-
     def open(self):
         self.stats = self.linter.add_stats(undocumented_module=0,
                                            undocumented_function=0,
                                            undocumented_method=0,
                                            undocumented_class=0)
+
     @utils.check_messages('missing-docstring', 'empty-docstring')
     def visit_module(self, node):
         self._check_docstring('module', node)
@@ -1383,10 +1615,7 @@ class DocStringChecker(_BasicChecker):
         if docstring is None:
             if not report_missing:
                 return
-            if node.body:
-                lines = node.body[-1].lineno - node.body[0].lineno + 1
-            else:
-                lines = 0
+            lines = get_node_last_lineno(node) - node.lineno
 
             if node_type == 'module' and not lines:
                 # If the module has no body, there's no reason
@@ -1423,6 +1652,7 @@ class PassChecker(_BasicChecker):
                       'Used when a "pass" statement that can be avoided is '
                       'encountered.'),
            }
+
     @utils.check_messages('unnecessary-pass')
     def visit_pass(self, node):
         if len(node.parent.child_sequence(node)) > 1:
@@ -1446,7 +1676,7 @@ class LambdaForComprehensionChecker(_BasicChecker):
 
     @utils.check_messages('deprecated-lambda')
     def visit_call(self, node):
-        """visit a CallFunc node, check if map or filter are called with a
+        """visit a Call node, check if map or filter are called with a
         lambda
         """
         if not node.args:
@@ -1588,7 +1818,6 @@ class ComparisonChecker(_BasicChecker):
         self.add_message('unidiomatic-typecheck', node=node)
 
 
-
 def register(linter):
     """required method to auto register this checker"""
     linter.register_checker(BasicErrorChecker(linter))
