@@ -1,3 +1,14 @@
+# -*- coding: utf-8 -*-
+# Copyright (c) 2014-2017 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2014 Michal Nowikowski <godfryd@gmail.com>
+# Copyright (c) 2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
+# Copyright (c) 2015 Pavel Roskin <proski@gnu.org>
+# Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
+# Copyright (c) 2016-2017 Pedro Algarvio <pedro@algarvio.me>
+# Copyright (c) 2016 Alexander Todorov <atodorov@otb.bg>
+# Copyright (c) 2017 Łukasz Rogalski <rogalski.91@gmail.com>
+# Copyright (c) 2017 Mikhail Fesenko <proggga@gmail.com>
+
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
 
@@ -5,25 +16,34 @@
 """
 
 import os
-import re
 import tokenize
-import string
-import sys
+import re
 
 try:
     import enchant
+    from enchant.tokenize import (get_tokenizer,
+                                  Chunker,
+                                  Filter,
+                                  EmailFilter,
+                                  URLFilter,
+                                  WikiWordFilter)
 except ImportError:
     enchant = None
+    # pylint: disable=old-style-class,no-init
+    class Filter:
+        def _skip(self, word):
+            raise NotImplementedError
+
+    class Chunker:
+        pass
+
+
 import six
 
 from pylint.interfaces import ITokenChecker, IAstroidChecker
 from pylint.checkers import BaseTokenChecker
 from pylint.checkers.utils import check_messages
-
-if sys.version_info[0] >= 3:
-    maketrans = str.maketrans
-else:
-    maketrans = string.maketrans
+from pylint.utils import safe_decode
 
 if enchant is not None:
     br = enchant.Broker()
@@ -37,7 +57,90 @@ else:
     dict_choices = ['']
     instr = " To make it working install python-enchant package."
 
-table = maketrans("", "")
+
+class WordsWithDigigtsFilter(Filter):
+    """Skips words with digits.
+    """
+
+    def _skip(self, word):
+        for char in word:
+            if char.isdigit():
+                return True
+        return False
+
+
+class WordsWithUnderscores(Filter):
+    """Skips words with underscores.
+
+    They are probably function parameter names.
+    """
+    def _skip(self, word):
+        return '_' in word
+
+
+class CamelCasedWord(Filter):
+    r"""Filter skipping over camelCasedWords.
+    This filter skips any words matching the following regular expression:
+
+           ^([a-z]\w+[A-Z]+\w+)
+
+    That is, any words that are camelCasedWords.
+    """
+    _pattern = re.compile(r"^([a-z]+([\d]|[A-Z])(?:\w+)?)")
+
+    def _skip(self, word):
+        return bool(self._pattern.match(word))
+
+
+class SphinxDirectives(Filter):
+    r"""Filter skipping over Sphinx Directives.
+    This filter skips any words matching the following regular expression:
+
+           ^:([a-z]+):`([^`]+)(`)?
+
+    That is, for example, :class:`BaseQuery`
+    """
+    # The final ` in the pattern is optional because enchant strips it out
+    _pattern = re.compile(r"^:([a-z]+):`([^`]+)(`)?")
+
+    def _skip(self, word):
+        return bool(self._pattern.match(word))
+
+
+class ForwardSlashChunkder(Chunker):
+    '''
+    This chunker allows splitting words like 'before/after' into 'before' and 'after'
+    '''
+    def next(self):
+        while True:
+            if not self._text:
+                raise StopIteration()
+            if '/' not in self._text:
+                text = self._text
+                self._offset = 0
+                self._text = ''
+                return (text, 0)
+            pre_text, post_text = self._text.split('/', 1)
+            self._text = post_text
+            self._offset = 0
+            if not pre_text or not post_text or \
+                    not pre_text[-1].isalpha() or not post_text[0].isalpha():
+                self._text = ''
+                self._offset = 0
+                return (pre_text + '/' + post_text, 0)
+            return (pre_text, 0)
+
+    def _next(self):
+        while True:
+            if '/' not in self._text:
+                return (self._text, 0)
+            pre_text, post_text = self._text.split('/', 1)
+            if not pre_text or not post_text:
+                break
+            if not pre_text[-1].isalpha() or not post_text[0].isalpha():
+                raise StopIteration()
+            self._text = pre_text + ' ' + post_text
+        raise StopIteration()
 
 
 class SpellingChecker(BaseTokenChecker):
@@ -80,6 +183,10 @@ class SpellingChecker(BaseTokenChecker):
                           'indicated private dictionary in '
                           '--spelling-private-dict-file option instead of '
                           'raising a message.'}),
+               ('max-spelling-suggestions',
+                {'default': 4, 'type': 'int', 'metavar': 'N',
+                 'help': 'Limits count of emitted suggestions for '
+                         'spelling mistakes'}),
               )
 
     def open(self):
@@ -113,10 +220,15 @@ class SpellingChecker(BaseTokenChecker):
         if self.config.spelling_store_unknown_words:
             self.unknown_words = set()
 
-        # Prepare regex for stripping punctuation signs from text.
-        # ' and _ are treated in a special way.
-        puncts = string.punctuation.replace("'", "").replace("_", "")
-        self.punctuation_regex = re.compile('[%s]' % re.escape(puncts))
+        self.tokenizer = get_tokenizer(dict_name,
+                                       chunkers=[ForwardSlashChunkder],
+                                       filters=[EmailFilter,
+                                                URLFilter,
+                                                WikiWordFilter,
+                                                WordsWithDigigtsFilter,
+                                                WordsWithUnderscores,
+                                                CamelCasedWord,
+                                                SphinxDirectives])
         self.initialized = True
 
     def close(self):
@@ -124,81 +236,68 @@ class SpellingChecker(BaseTokenChecker):
             self.private_dict_file.close()
 
     def _check_spelling(self, msgid, line, line_num):
-        line2 = line.strip()
-        # Replace ['afadf with afadf (but preserve don't)
-        line2 = re.sub("'([^a-zA-Z]|$)", " ", line2)
-        # Replace afadf'] with afadf (but preserve don't)
-        line2 = re.sub("([^a-zA-Z]|^)'", " ", line2)
-        # Replace punctuation signs with space e.g. and/or -> and or
-        line2 = self.punctuation_regex.sub(' ', line2)
+        original_line = line
+        if line.strip().startswith('#'):
+            line = line.strip()[1:]
+            starts_with_comment = True
+        else:
+            starts_with_comment = False
+        for word, _ in self.tokenizer(line.strip()):
+            if six.PY2:
+                lower_cased_word = word.lower()
+            else:
+                lower_cased_word = word.casefold()
 
-        words = []
-        for word in line2.split():
-            # Skip words with digits.
-            if len(re.findall(r"\d", word)) > 0:
-                continue
-
-            # Skip words with mixed big and small letters,
-            # they are probaly class names.
-            if (len(re.findall("[A-Z]", word)) > 0 and
-                    len(re.findall("[a-z]", word)) > 0 and
-                    len(word) > 2):
-                continue
-
-            # Skip words with _ - they are probably function parameter names.
-            if word.count('_') > 0:
-                continue
-
-            words.append(word)
-
-        # Go through words and check them.
-        for word in words:
             # Skip words from ignore list.
-            if word in self.ignore_list:
+            if word in self.ignore_list or lower_cased_word in self.ignore_list:
                 continue
 
-            orig_word = word
-            word = word.lower()
-
             # Strip starting u' from unicode literals and r' from raw strings.
-            if (word.startswith("u'") or
-                    word.startswith('u"') or
-                    word.startswith("r'") or
-                    word.startswith('r"')) and len(word) > 2:
+            if word.startswith(("u'", 'u"', "r'", 'r"')) and len(word) > 2:
                 word = word[2:]
+                lower_cased_word = lower_cased_word[2:]
 
             # If it is a known word, then continue.
             try:
+                if self.spelling_dict.check(lower_cased_word):
+                    # The lower cased version of word passed spell checking
+                    continue
+
+                # If we reached this far, it means there was a spelling mistake.
+                # Let's retry with the original work because 'unicode' is a
+                # spelling mistake but 'Unicode' is not
                 if self.spelling_dict.check(word):
                     continue
             except enchant.errors.Error:
-                # this can only happen in docstrings, not comments
                 self.add_message('invalid-characters-in-docstring',
                                  line=line_num, args=(word,))
                 continue
 
             # Store word to private dict or raise a message.
             if self.config.spelling_store_unknown_words:
-                if word not in self.unknown_words:
-                    self.private_dict_file.write("%s\n" % word)
-                    self.unknown_words.add(word)
+                if lower_cased_word not in self.unknown_words:
+                    self.private_dict_file.write("%s\n" % lower_cased_word)
+                    self.unknown_words.add(lower_cased_word)
             else:
-                # Present up to 4 suggestions.
-                # TODO: add support for customising this.
-                suggestions = self.spelling_dict.suggest(word)[:4]
+                # Present up to N suggestions.
+                suggestions = self.spelling_dict.suggest(word)
+                del suggestions[self.config.max_spelling_suggestions:]
 
-                m = re.search(r"(\W|^)(%s)(\W|$)" % word, line.lower())
+                m = re.search(r"(\W|^)(%s)(\W|$)" % word, line)
                 if m:
                     # Start position of second group in regex.
                     col = m.regs[2][0]
                 else:
-                    col = line.lower().index(word)
+                    col = line.index(word)
+
+                if starts_with_comment:
+                    col += 1
                 indicator = (" " * col) + ("^" * len(word))
 
                 self.add_message(msgid, line=line_num,
-                                 args=(orig_word, line,
+                                 args=(word, original_line,
                                        indicator,
-                                       "' or '".join(suggestions)))
+                                       "'{0}'".format("' or '".join(suggestions))))
 
     def process_tokens(self, tokens):
         if not self.initialized:
@@ -207,6 +306,12 @@ class SpellingChecker(BaseTokenChecker):
         # Process tokens and look for comments.
         for (tok_type, token, (start_row, _), _, _) in tokens:
             if tok_type == tokenize.COMMENT:
+                if start_row == 1 and token.startswith('#!/'):
+                    # Skip shebang lines
+                    continue
+                if token.startswith('# pylint:'):
+                    # Skip pylint enable/disable comments
+                    continue
                 self._check_spelling('wrong-spelling-in-comment',
                                      token, start_row)
 
@@ -239,8 +344,7 @@ class SpellingChecker(BaseTokenChecker):
         start_line = node.lineno + 1
         if six.PY2:
             encoding = node.root().file_encoding
-            docstring = docstring.decode(encoding or sys.getdefaultencoding(),
-                                         'replace')
+            docstring = safe_decode(docstring, encoding, 'replace')
 
         # Go through lines of docstring
         for idx, line in enumerate(docstring.splitlines()):
