@@ -1,10 +1,26 @@
+# -*- coding: utf-8 -*-
 # Copyright (c) 2006-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
-# Copyright (c) 2011-2014 Google, Inc.
-# Copyright (c) 2013-2016 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2009 Mads Kiilerich <mads@kiilerich.com>
+# Copyright (c) 2010 Daniel Harding <dharding@gmail.com>
+# Copyright (c) 2011-2014, 2017 Google, Inc.
+# Copyright (c) 2012 FELD Boris <lothiraldan@gmail.com>
+# Copyright (c) 2013-2017 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2014 Michal Nowikowski <godfryd@gmail.com>
-# Copyright (c) 2015 Radu Ciorba <radu@devrandom.ro>
+# Copyright (c) 2014 Brett Cannon <brett@python.org>
+# Copyright (c) 2014 Ricardo Gemignani <ricardo.gemignani@gmail.com>
+# Copyright (c) 2014 Arun Persaud <arun@nubati.net>
 # Copyright (c) 2015 Dmitry Pribysh <dmand@yandex.ru>
+# Copyright (c) 2015 Radu Ciorba <radu@devrandom.ro>
+# Copyright (c) 2015 Simu Toni <simutoni@gmail.com>
+# Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
+# Copyright (c) 2016-2017 Derek Gustafson <degustaf@gmail.com>
+# Copyright (c) 2016-2017 Łukasz Rogalski <rogalski.91@gmail.com>
+# Copyright (c) 2016 Grant Welch <gwelch925+github@gmail.com>
 # Copyright (c) 2016 Ashley Whetter <ashley@awhetter.co.uk>
+# Copyright (c) 2016 Jakub Wilk <jwilk@jwilk.net>
+# Copyright (c) 2017 hippo91 <guillaume.peillex@gmail.com>
+# Copyright (c) 2017 Dan Garrette <dhgarrette@gmail.com>
+# Copyright (c) 2017 Ville Skyttä <ville.skytta@iki.fi>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
@@ -44,12 +60,13 @@ def _is_from_future_import(stmt, name):
     try:
         module = stmt.do_import_module(stmt.modname)
     except astroid.AstroidBuildingException:
-        return
+        return None
 
     for local_node in module.locals.get(name, []):
         if (isinstance(local_node, astroid.ImportFrom)
                 and local_node.modname == FUTURE):
             return True
+    return None
 
 
 def in_for_else_branch(parent, stmt):
@@ -203,6 +220,7 @@ def _find_frame_imports(name, frame):
                     return True
             elif import_name and import_name == name:
                 return True
+    return None
 
 
 def _import_name_is_global(stmt, global_names):
@@ -281,7 +299,7 @@ MSGS = {
 
     'W0621': ('Redefining name %r from outer scope (line %s)',
               'redefined-outer-name',
-              'Used when a variable\'s name hide a name defined in the outer \
+              'Used when a variable\'s name hides a name defined in the outer \
               scope.'),
     'W0622': ('Redefining built-in %r',
               'redefined-builtin',
@@ -356,7 +374,7 @@ class VariablesChecker(BaseChecker):
                           'end with one of those strings.'}
                ),
                ("redefining-builtins-modules",
-                {'default': ('six.moves', 'future.builtins'), 'type': 'csv',
+                {'default': ('six.moves', 'past.builtins', 'future.builtins'), 'type': 'csv',
                  'metavar': '<comma separated list>',
                  'help': 'List of qualified module names which can have objects '
                          'that can redefine builtins.'}
@@ -460,7 +478,7 @@ class VariablesChecker(BaseChecker):
                 elt_name = next(elt.infer())
             except astroid.InferenceError:
                 continue
-            if elt_name is astroid.YES:
+            if elt_name is astroid.Uninferable:
                 continue
             if not elt_name.parent:
                 continue
@@ -656,6 +674,12 @@ class VariablesChecker(BaseChecker):
         # Ignore some special names specified by user configuration.
         if self._is_name_ignored(stmt, name):
             return
+        # Ignore names that were added dynamically to the Function scope
+        if (isinstance(node, astroid.FunctionDef)
+                and name == '__class__'
+                and len(node.locals['__class__']) == 1
+                and isinstance(node.locals['__class__'][0], astroid.ClassDef)):
+            return
 
         # Ignore names imported by the global statement.
         # FIXME: should only ignore them if it's assigned latter
@@ -1006,6 +1030,7 @@ class VariablesChecker(BaseChecker):
             # name node from a astroid built from live code, skip
             assert not stmt.root().file.endswith('.py')
             return
+
         name = node.name
         frame = stmt.scope()
         # if the name node is used as a function default argument's value or as
@@ -1215,7 +1240,7 @@ class VariablesChecker(BaseChecker):
                 break
             try:
                 module = next(module.getattr(name)[0].infer())
-                if module is astroid.YES:
+                if module is astroid.Uninferable:
                     return None
             except astroid.NotFoundError:
                 if module.name in self._ignored_modules:
